@@ -8,13 +8,11 @@ import io
 import distutils.errors
 import itertools
 import stat
-
-import six
-from six.moves import map, filter, filterfalse
+from more_itertools import unique_everseen
 
 try:
     from setuptools.lib2to3_ex import Mixin2to3
-except ImportError:
+except Exception:
 
     class Mixin2to3:
         def run_2to3(self, files, doctests=True):
@@ -73,9 +71,6 @@ class build_py(orig.build_py, Mixin2to3):
         return orig.build_py.__getattr__(self, attr)
 
     def build_module(self, module, module_file, package):
-        if six.PY2 and isinstance(package, six.string_types):
-            # avoid errors on Python 2 when unicode is passed (#190)
-            package = package.split('.')
         outfile, copied = orig.build_py.build_module(self, module, module_file,
                                                      package)
         if copied:
@@ -220,7 +215,7 @@ class build_py(orig.build_py, Mixin2to3):
             if fn not in bad
         )
         # ditch dupes
-        return list(_unique_everseen(keepers))
+        return list(unique_everseen(keepers))
 
     @staticmethod
     def _get_platform_patterns(spec, package, src_dir):
@@ -241,25 +236,6 @@ class build_py(orig.build_py, Mixin2to3):
         )
 
 
-# from Python docs
-def _unique_everseen(iterable, key=None):
-    "List unique elements, preserving order. Remember all elements ever seen."
-    # unique_everseen('AAAABBBCCDAABBB') --> A B C D
-    # unique_everseen('ABBCcAD', str.lower) --> A B C D
-    seen = set()
-    seen_add = seen.add
-    if key is None:
-        for element in filterfalse(seen.__contains__, iterable):
-            seen_add(element)
-            yield element
-    else:
-        for element in iterable:
-            k = key(element)
-            if k not in seen:
-                seen_add(k)
-                yield element
-
-
 def assert_relative(path):
     if not os.path.isabs(path):
         return path
